#!/usr/bin/env python

# adapted from DOI:10.1371/journal.pcbi.1003499.s001

import vtk, sys

WIDTH=1200 # size of the renderer
HEIGHT=900
NUMSIDES = 16 # smoothness of cylinder and sphere visualizations

def loadTree ( Filename, Nodes, Edges ):
    if Filename.endswith (".tree" ) or Filename.endswith (".txt" ):
        treefile = open ( Filename, mode='r' )
    else:
        print Filename + ": cannot detect file name ending"
        return

    treelines = treefile.readlines()
    treefile.close()

    # load number of nodes, ignoring comments
    while ( treelines[0].startswith ("#") ):
        treelines.pop(0)
    numNodes = int ( treelines[0] )
    treelines.pop(0)

    while ( treelines[0].startswith ("#") ):
        treelines.pop(0)

    # load node coordinates
    loadedNodes = []
    for i in range ( numNodes ):
        numInLine = treelines[0].split()
        loadedNodes.append ( [ float ( numInLine[1] ), float ( numInLine[2] ), float ( numInLine[3] ) ] )
        treelines.pop(0)

    # load number of edges, igoring comments
    while ( treelines[0].startswith ("#") ):
        treelines.pop(0)
    numEdges = int ( treelines[0] )
    treelines.pop(0)

    while ( treelines[0].startswith ("#") ):
        treelines.pop(0)

    # load edge data
    loadedEdges = []
    for i in range ( numEdges ):
        numInLine = treelines[0].split()
        loadedEdges.append ( [ int ( numInLine[1] ), int ( numInLine[2] ), int ( numInLine[3] ) , float ( numInLine[ 5 + int ( numInLine[4] ) ] ) ] ) # initial node, terminal node, parent edge, radius decrease factor
        treelines.pop(0)

    Nodes[:] = loadedNodes
    # compute edge radii as product of decrease factors up to root
    Edges[:] = []
    for e in loadedEdges:
        pathIndex = e[2];
        radius = e[3]
        while ( pathIndex != 2147483646 ):
            radius = radius * loadedEdges[ pathIndex ][3];
            pathIndex = loadedEdges [ pathIndex ][2];
        Edges.append ( [ e[0], e[1], radius ] );


def visualizeTree ( Nodes, Edges, Renderer ):
    # for each of the edges ...
    for e in Edges:
        edgeAsLine = vtk.vtkLineSource()
        edgeAsLine.SetPoint1 ( Nodes[ e[0] ] );
        edgeAsLine.SetPoint2 ( Nodes[ e[1] ] );

        # ... that do not have length 0 ...
        if ( Nodes[ e[0] ] != Nodes[ e[1] ] ):
            # ... display a cylinder to represent the edge ...
            edgeAsCylind = vtk.vtkTubeFilter();
            edgeAsCylind.SetInput ( edgeAsLine.GetOutput() );
            edgeAsCylind.SetRadius ( e[2] );
            edgeAsCylind.CappingOn();
            edgeAsCylind.SetNumberOfSides(NUMSIDES);

            edgeMapper = vtk.vtkPolyDataMapper()
            edgeMapper.SetInput(edgeAsCylind.GetOutput())

            edgeActor = vtk.vtkActor()
            edgeActor.SetMapper(edgeMapper)

            Renderer.AddActor(edgeActor)

            # ... and a sphere (half of which will be visible) at the end to obtain smoother transitions
            termNodeAsSphere = vtk.vtkSphereSource()
            termNodeAsSphere.SetCenter(Nodes[e[1]])
            termNodeAsSphere.SetRadius(e[2])
            termNodeAsSphere.LatLongTessellationOff()
            termNodeAsSphere.SetPhiResolution( NUMSIDES )
            termNodeAsSphere.SetThetaResolution( NUMSIDES )

            sphereMapper = vtk.vtkPolyDataMapper()
            sphereMapper.SetInput(termNodeAsSphere.GetOutput())

            sphereActor = vtk.vtkActor()
            sphereActor.SetMapper(sphereMapper)

            Renderer.AddActor(sphereActor)

    # finally also display a sphere at the root node, if the tree was not empty
    if ( len(Edges) > 0 and len(Nodes) > 0 ):
        rootNodeAsSphere = vtk.vtkSphereSource()
        rootNodeAsSphere.SetCenter(Nodes[Edges[0][0]])
        rootNodeAsSphere.SetRadius(Edges[0][2])
        rootNodeAsSphere.LatLongTessellationOff()
        rootNodeAsSphere.SetPhiResolution( NUMSIDES )
        rootNodeAsSphere.SetThetaResolution( NUMSIDES )

        sphereMapper = vtk.vtkPolyDataMapper()
        sphereMapper.SetInput(rootNodeAsSphere.GetOutput())

        sphereActor = vtk.vtkActor()
        sphereActor.SetMapper(sphereMapper)

        Renderer.AddActor(sphereActor)

# set up rendering
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
renWin.SetSize(WIDTH,HEIGHT)

iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

Nodes = []
Edges = []

loadTree ( sys.argv[1], Nodes, Edges )

visualizeTree ( Nodes, Edges, ren )

iren.Initialize()
renWin.Render()
iren.Start()
